﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace TechnoAP.usbmca.common
{
    using System.IO;

    /// <summary>
    /// Summary for file
    /// </summary>
    public class file
    {
        public const long SEEK_1ST = 0;
        public const long SEEK_END = -1;

        /// <summary>
        /// Write from binary file
        /// </summary>
        public static bool WriteBinary(string savefile, long lngSeek, int nBytes, byte[] abBuffer)
        {
            FileStream fsSave = null;
            bool blnResult = true;

            try
            {
                // Open the write destination file
                fsSave = File.Open(savefile, FileMode.OpenOrCreate, FileAccess.Write, FileShare.ReadWrite);

                if (lngSeek == SEEK_END)
                {
                    // Write from the end of the file
                    fsSave.Seek(0, SeekOrigin.End);
                }
                else
                {
                    // Relative position from the beginning
                    fsSave.Seek(lngSeek, SeekOrigin.Begin);
                }

                // writing
                fsSave.Write(abBuffer, 0, nBytes);
            }
            catch (Exception ex)
            {
                // Failure
                blnResult = false;
            }

            if (fsSave != null)
            {
                fsSave.Close();
            }

            return blnResult;

        }

        /// <summary>
        /// Read from binary file
        /// </summary>
        public static bool ReadBinary(string savefile, long lngSeek, int nBytes, byte[] abBuffer)
        {
            FileStream fsSave = null;
            bool blnResult = true;

            try
            {
                fsSave = File.Open(savefile, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
                fsSave.Seek(lngSeek, SeekOrigin.Begin);             // Move to read position
                int nLength = fsSave.Read(abBuffer, 0, nBytes);     // Reading binary data

                if(nLength != nBytes)
                {
                    // Read size mismatch -> failure
                    blnResult = false;
                }
            }
            catch (Exception ex)
            {
                // Error occurred
                blnResult = false;
            }

            if (fsSave != null)
            {
                fsSave.Close();
            }

            return blnResult;

        }

        /// <summary>
        /// Preservation of histogram data
        /// </summary>
        public static Exception SaveHistogram(string savefile, data.histogram histgram)
        {
            string tmpLine = null;
            Exception exResult = null;
            StreamWriter swFile = null;

            try
            {
                try { System.IO.File.Delete(savefile); }
                catch (Exception ex) { /* nothing */ }

                uint[][] auHistogram = histgram.mauHistogram;

                // Calculation of summation
                long[] alngTotal = new long[auHistogram.GetLength(0)];
                Array.Clear(alngTotal, 0, alngTotal.Length);

                for(int i = 0; i < alngTotal.Length; i++)
                {
                    for(int j = 0; j < auHistogram[i].Length; j++)
                    {
                        alngTotal[i] += (long)(auHistogram[i][j]);
                    }
                }

                swFile = new StreamWriter(savefile, true, Encoding.Default);

                // save header
                swFile.WriteLine("[system]");
                swFile.WriteLine("save time,{0}", DateTime.Now.ToString());

                // save ADC gain
                tmpLine = "ADG";

                for (int i = 0; i < histgram.mauADG.Length; i++) {
                    int nAdgMaxCh = constant.Max.GetAdgMaxCh(histgram.mauADG[i]);
                    tmpLine += string.Format(",{0}", nAdgMaxCh);
                }

                swFile.WriteLine(tmpLine);

                // outoput total count
                tmpLine = "total count";

                for(int i = 0; i < alngTotal.Length; i++)
                {
                    tmpLine += ("," + alngTotal[i].ToString());
                }

                swFile.WriteLine(tmpLine);

                // save histogram data
                swFile.Write(swFile.NewLine);
                swFile.WriteLine("[data]");
                swFile.WriteLine("#ch,CH1,CH2,CH3,CH4");

                for (int i = 0; i < auHistogram[0].Length; i++)
                {
                    tmpLine = string.Format("{0}", i);

                    for (int j = 0; j < auHistogram.Length; j++)
                    {
                        tmpLine += string.Format(",{0}", auHistogram[j][i]);
                    }

                    swFile.WriteLine(tmpLine);
                }
            }
            catch (Exception ex)
            {
                exResult = ex;
            }

            if(swFile != null)
            {
                swFile.Close();
            }

            return exResult;

        }
    }

}
