﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace TechnoAP.usbmca.constant
{
    /// <summary>
    /// Summary for constant
    /// </summary>
    public class Max
    {
        public const int CH = 4;                        // CH count
        public const int HistogramRecieveCount = 32;    // number of times to receive histogram data(per 1ch)
        public const int Resolution = 16384;            // resolution

        public const int ListDataLengthByte = 8;        // list data size(1 unit)

        private const uint AdcGainEnd = (uint)(AdcGain.CH_512); // end of ADG code

        /// <summary>
        /// Acquisition of maximum ADC gain value(ch)
        /// </summary>
        static public int GetAdgMaxCh(uint uAdg)
        {
            int result = 0;

            for(uint i = 0; i <= AdcGainEnd; i++)
            {
                if(uAdg == i)
                {
                    int n = (int)(Math.Pow(2.0, (double)(i)));
                    result = Resolution / n;
                    break;
                }
            }

            return result;
        }
    }

    /// <summary>
    /// Summary for Size
    /// </summary>
    public class Size
    {
        public const int HistogramOneChSize = sizeof(int);  // size(byte) per 1ch
        public const int HistogramOnceCh = 512;             // number of reception channels per reception
        public const int HistogramOnceByte = HistogramOneChSize * HistogramOnceCh;      // one reception size
        public const int HistogramMaxCh = HistogramOnceCh * Max.HistogramRecieveCount;  // receive size of 1CH

        public const int ListDataBlock = 8;     // list data size
    }

    /// <summary>
    /// Summary for Unit
    /// </summary>
    public class Unit
    {
        public const long CountPerDigitNsec = 40;   /* count/digit(nsec) */
    }

    /// <summary>
    /// Summary for 
    /// </summary>
    public class Command
    {
        // A single command
        public const string MODW = "MODW";      // measurement mode(histogram or list)
        public const string MMDW = "MMDW";      // time mode(real or live)
        public const string MT0W = "MT0W";      // measurement time(msb 32bit)
        public const string MT1W = "MT1W";      // measurement time(lsb 32bit)
        public const string PDSW = "PDSW";      // peak search
        public const string AQSW = "AQSW";      // start measurement
        public const string AQEW = "AQEW";      // stop  measurement
        public const string CLRW = "CLRW";      // clear measurement data
        public const string HCHW = "HCHW";      // change histogram CH number
        public const string LISR = "LISR";      // request list data
        public const string VER0 = "VER0";      // version1
        public const string VER1 = "VER1";      // version2

        public const string HIXX = "HI{0:X2}";  // request histogram data block number
        public const string LISA = "LISA";      // list data size(answer)
        
        public const string STUW = "STUW";      // request status

                                              /* command of CH unit */
        public const string ADG = "ADG";        // ADC gain
        public const string THR = "THR";        // threshold(ch)
        public const string LLD = "LLD";        // energy lower discrimination level(ch)
        public const string ULD = "ULD";        // energy upper discrimination level(ch)
        public const string OFS = "OFS";        // offset(ch)
        public const string IOF = "IOF";        // unuse...
        public const string IFG = "IFG";        // unuse...

        /// <summary>
        /// Command comparison
        /// </summary>
        public static bool isChCommand(string command1, string command2)
        {
            bool blnResult = false;

            if(command1 == command2)
            {
                blnResult = true;
            }

            return blnResult;
        }
    }

    /// <summary>
    /// Summary for CH_TYPE(Enum)
    /// </summary>
    public enum CH_TYPE : int
    {
        Common = 0, // 共通
        CH1 = 1,    // CH1
        CH2 = 2,    // CH2
        CH3 = 3,    // CH3
        CH4 = 4     // CH4
    }

    /// <summary>
    /// Summary for DataMode(Enum)
    /// </summary>
    public enum DataMode : uint
    {
        Histogram = 0,
        List = 1
    }

    /// <summary>
    /// Summary for TimeMode(Enum)
    /// </summary>
    public enum TimeMode : uint
    {
        Real = 0,
        Live = 1
    }
    
    /// <summary>
    /// Summary for PdsMode(Enum)
    /// </summary>
    public enum PdsMode : uint
    {
        Abs = 0,
        Fast = 1
    }

    /// <summary>
    /// Summary for AdcGain(Enum)
    /// </summary>
    public enum AdcGain : uint
    {
        CH_16K = 0, // 16384ch
        CH_08K = 1, //  8192ch
        CH_04K = 2, //  4096ch
        CH_02K = 3, //  2048ch
        CH_01K = 4, //  1024ch
        CH_512 = 5  //   512ch
    }
}
