﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TechnoAP.usbmca.device
{
    using TechnoAP.usbmca;

    /// <summary>
    /// Summary for config
    /// </summary>
    public class config
    {
        public device mDevice = null;

        public uint[] manADG = new uint[constant.Max.CH];
        public uint[] manTHR = new uint[constant.Max.CH];
        public uint[] manLLD = new uint[constant.Max.CH];
        public uint[] manULD = new uint[constant.Max.CH];
        public uint[] manOFS = new uint[constant.Max.CH];
        public uint[] manIOF = new uint[constant.Max.CH];
        public uint[] manIFG = new uint[constant.Max.CH];
        public TechnoAP.usbmca.constant.PdsMode muPDS;

        /// <summary>
        /// constractor
        /// </summary>
        public config()
        {
            mDevice = null;
        }

        public config(device clsDevice)
        {
            mDevice = clsDevice;
        }

        // 配列型
        public void SetADG(uint[] anValue) { CopyConfigData(ref anValue, ref manADG); }
        public void SetTHR(uint[] anValue) { CopyConfigData(ref anValue, ref manTHR); }
        public void SetLLD(uint[] anValue) { CopyConfigData(ref anValue, ref manLLD); }
        public void SetULD(uint[] anValue) { CopyConfigData(ref anValue, ref manULD); }
        public void SetOFS(uint[] anValue) { CopyConfigData(ref anValue, ref manOFS); }
        public void SetIOF(uint[] anValue) { CopyConfigData(ref anValue, ref manIOF); }
        public void SetIFG(uint[] anValue) { CopyConfigData(ref anValue, ref manIFG); }

        // 各CH型
        public void SetADG(uint uCh1 = 0, uint uCh2 = 0, uint uCh3 = 0, uint uCh4 = 0) { CopyConfigData(ref manADG, uCh1, uCh2, uCh3, uCh4); }
        public void SetTHR(uint uCh1 = 0, uint uCh2 = 0, uint uCh3 = 0, uint uCh4 = 0) { CopyConfigData(ref manTHR, uCh1, uCh2, uCh3, uCh4); }
        public void SetLLD(uint uCh1 = 0, uint uCh2 = 0, uint uCh3 = 0, uint uCh4 = 0) { CopyConfigData(ref manLLD, uCh1, uCh2, uCh3, uCh4); }
        public void SetULD(uint uCh1 = 0, uint uCh2 = 0, uint uCh3 = 0, uint uCh4 = 0) { CopyConfigData(ref manULD, uCh1, uCh2, uCh3, uCh4); }
        public void SetOFS(uint uCh1 = 0, uint uCh2 = 0, uint uCh3 = 0, uint uCh4 = 0) { CopyConfigData(ref manOFS, uCh1, uCh2, uCh3, uCh4); }
        public void SetIOF(uint uCh1 = 0, uint uCh2 = 0, uint uCh3 = 0, uint uCh4 = 0) { CopyConfigData(ref manIOF, uCh1, uCh2, uCh3, uCh4); }
        public void SetIFG(uint uCh1 = 0, uint uCh2 = 0, uint uCh3 = 0, uint uCh4 = 0) { CopyConfigData(ref manIFG, uCh1, uCh2, uCh3, uCh4); }

        public void SetPDS(TechnoAP.usbmca.constant.PdsMode uPDS) { muPDS = uPDS; }

        /// <summary>
        /// Copying parameters (array)
        /// </summary>
        private static void CopyConfigData(ref uint[] source, ref uint[] destination)
        {
            Array.Copy(source, destination, destination.Length);
        }

        /// <summary>
        /// Copying parameters
        /// </summary>
        private static void CopyConfigData(ref uint[] destination, uint uCh1 = 0, uint uCh2 = 0, uint uCh3 = 0, uint uCh4 = 0)
        {
            uint[] anConfig = new uint[constant.Max.CH];

            anConfig[0] = uCh1;
            anConfig[1] = uCh2;
            anConfig[2] = uCh3;
            anConfig[3] = uCh4;
            CopyConfigData(ref anConfig, ref destination);
        }

        /// <summary>
        /// Writing measurement parameters
        /// </summary>
        public bool Write()
        {
            bool[] astResult = new bool[32];
            bool blnResult;

            blnResult = true;

            // initial, return value
            for (int i = 0; i < astResult.Length; i++)
            {
                astResult[i] = true;
            }

            // writing config, CH unit
            for(int i = 0; i < constant.Max.CH; i++)
            {
                int index = 0;
                uint uOutput = 0;

                astResult[index++] = mDevice.OrderCommand(constant.Command.ADG, (i + 1), manADG[i], ref uOutput);
                astResult[index++] = mDevice.OrderCommand(constant.Command.THR, (i + 1), manTHR[i], ref uOutput);
                astResult[index++] = mDevice.OrderCommand(constant.Command.LLD, (i + 1), manLLD[i], ref uOutput);
                astResult[index++] = mDevice.OrderCommand(constant.Command.ULD, (i + 1), manULD[i], ref uOutput);
                astResult[index++] = mDevice.OrderCommand(constant.Command.OFS, (i + 1), manOFS[i], ref uOutput);
                astResult[index++] = mDevice.OrderCommand(constant.Command.PDSW, 0, (uint)muPDS, ref uOutput);
#if false
                astResult[index++] = mDevice.OrderCommand(constant.Command.IOF, (i + 1), manIOF[i], ref uOutput);
                astResult[index++] = mDevice.OrderCommand(constant.Command.IFG, (i + 1), manIFG[i], ref uOutput);
#endif

                // check error...
                for (int j = 0; j < index; j++)
                {
                    if (astResult[j] == false)
                    {
                        // There is an error
                        blnResult = false;  // error ON.
                        break;
                    }
                }
            }

            return blnResult;

        }
    }
}
