﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;


namespace usbmca_list
{
    using FTD2XX_NET;

    /// <summary>
    /// Summary for formMain
    /// </summary>
    public partial class formMain : Form
    {
        public TechnoAP.usbmca.device.device mDevice = new TechnoAP.usbmca.device.device();
        public TechnoAP.usbmca.device.status mStatus = null;

        public long mlngMeasurementTime = (3 * 1000);               // measurement time(msec)
        public long mlngReadedBytes = 0;

        public string mstrSaveFile = "C:\\Data\\list.bin";      // save file
        public string mstrDeviceSerialNumber = "APG7400AA";     // serial number

        public TechnoAP.usbmca.data.list mList = new TechnoAP.usbmca.data.list();

        public uint[] mauAdg = new uint[TechnoAP.usbmca.constant.Max.CH];   // ADC Gain

        public formMain()
        {
            InitializeComponent();
            //
            //TODO: Add the constructor code here
            //
            tmrMeasurement.Enabled = false;
        }

        /// <summary>
        /// Form load event
        /// </summary>
        private void formMain_Load(object sender, EventArgs e)
        {
            bool blnButton = false;

            // Connection with USB-MCA
            bool blnResult0 = ConnectDevice(mstrDeviceSerialNumber);

            if (blnResult0 == true)
            {
                // Writing measurement parameters
                bool blnResult1 = WriteConfig();

                if (blnResult1 == true)
                {
                    blnButton = true;
                }
                else
                {
                    // error
                    MessageBox.Show("device config write error...", "error...",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            btnStart.Enabled = blnButton;
            btnStop.Enabled = blnButton;
        }

        /// <summary>
        /// Device(USB-MCA) connection procedure
        /// </summary>
        private bool ConnectDevice(string strSerialNum)
        {
            bool blnResult = true;

            // Connection with USB-MCA by serial number
            FTDI.FT_STATUS ftStatus = mDevice.OpenBySerialNumber(strSerialNum);

            if (ftStatus == FTDI.FT_STATUS.FT_OK)
            {
                FTDI.FT_STATUS[] astResult = new FTDI.FT_STATUS[8];

                for (int i = 0; i < astResult.Length; i++)
                {
                    astResult[i] = FTDI.FT_STATUS.FT_OK;
                }

                //////////Initialization request for USB-MCA//////////
                int index = 0;
                astResult[index++] = mDevice.ResetDevice();
                astResult[index++] = mDevice.SetBitMode(255, FTDI.FT_BIT_MODES.FT_BIT_MODE_RESET);
                astResult[index++] = mDevice.SetLatency(2);
                astResult[index++] = mDevice.SetTimeouts(100, 100);
                astResult[index++] = mDevice.SetFlowControl(FTDI.FT_FLOW_CONTROL.FT_FLOW_RTS_CTS, 0, 0);
                astResult[index++] = mDevice.Purge(FTDI.FT_PURGE.FT_PURGE_RX | FTDI.FT_PURGE.FT_PURGE_TX);
                System.Threading.Thread.Sleep(500);     // wait accompanying reset
                /////////////////////////////////

                // Display serial number
                txtConnectedDevice.Text = mstrDeviceSerialNumber;

                for (int i = 0; i < astResult.Length; i++)
                {
                    if (astResult[i] != FTDI.FT_STATUS.FT_OK)
                    {
                        // There was an error in initial processing
                        MessageBox.Show("device initial setting error...", "error...", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        txtConnectedDevice.Text = string.Empty;
                        break;
                    }
                }
            }
            else
            {
                // device open error...
                MessageBox.Show("device connect error...", "error...",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                blnResult = false;
            }

            return blnResult;
        }

        /// <summary>
        /// Device(USB-MCA) config write procedure
        /// </summary>
        private bool WriteConfig()
        {
            uint[] auULD = new uint[mauAdg.Length];

            for (int i = 0; i < mauAdg.Length; i++)
            {
                mauAdg[i] = (uint)TechnoAP.usbmca.constant.AdcGain.CH_04K;   // 4096ch
                auULD[i] = (uint)TechnoAP.usbmca.constant.Max.GetAdgMaxCh(mauAdg[i]) - 1;
            }

            ////////////Writing measurement parameters/////////////////////////
            TechnoAP.usbmca.device.config tmpConfig = new TechnoAP.usbmca.device.config(mDevice);
            tmpConfig.SetADG(mauAdg);           // ADC Gain
            tmpConfig.SetTHR(80, 80, 80, 80);   // threshold
            tmpConfig.SetLLD(95, 95, 95, 95);   // lower level discri
            tmpConfig.SetULD(auULD);            // upper level discri
            tmpConfig.SetOFS(0, 0, 0, 0);       // offset
            tmpConfig.SetPDS(TechnoAP.usbmca.constant.PdsMode.Abs); // absolution
            bool blnResult = tmpConfig.Write();  // write config
            ///////////////////////////////////////////////////////////////////

            return blnResult;
        }

        /// <summary>
        /// Start button click event
        /// </summary>
        private void btnStart_Click(object sender, EventArgs e)
        {
            try
            {
                try { System.IO.File.Delete(mstrSaveFile); }
                catch(Exception ex) { /* nothging */ }

                mlngReadedBytes = 0;
                txtReadBytes.Text = mlngReadedBytes.ToString();

                mDevice.StopMeasurement();          // Measurement stop
                mDevice.ClearMeasurementData();     // Clear measurement data

                WriteConfig();      // Writing measurement parameters

                mDevice.SetMeasurementTimeMilliseconds(mlngMeasurementTime);                // Setting measurement time
                mDevice.SetMeasurementDataMode(TechnoAP.usbmca.constant.DataMode.List);     // list mode
                mDevice.SetMeasurementTimeMode(TechnoAP.usbmca.constant.TimeMode.Real);     // Real time mode

                // Start measurement
                mDevice.StartMeasurement();
                tmrMeasurement.Start();
                
                btnStart.Enabled = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Exception...",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Stop button click event
        /// </summary>
        private void btnStop_Click(object sender, EventArgs e)
        {
            try
            {
                tmrMeasurement.Stop();          // Stop the measurement timer
                mDevice.StopMeasurement();      // Measurement stop
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Exception...",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            btnStart.Enabled = true;
        }

        /// <summary>
        /// Measurement timer event
        /// </summary>
        private void tmrMeasurement_Tick(object sender, EventArgs e)
        {
            long lngMilliseconds = 0;

            try
            {
                mStatus = mDevice.ReadStatus();                             // Reading measurement status
                lngMilliseconds = mStatus.getElapsedTimeMilliseconds();     // elapsed time

                ReadListData();     // Read list data
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Exception...",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                btnStop_Click(sender, e);
            }

            if (lngMilliseconds >= mlngMeasurementTime)
            {
                ReadListData();                 // Measurement time reached
                btnStop_Click(sender, e);       // Processing at measurement stop
            }

            // Display of measurement elapsed time
            txtElapsedTime.Text = lngMilliseconds.ToString();
        }

        /// <summary>
        /// Procedure for reading list data
        /// </summary>
        private int ReadListData()
        {
            int nLength = 0;
            byte[] abListData = mDevice.ReadListData(ref nLength);

            if (nLength > 0)
            {
                // Display update of list data size
                mlngReadedBytes += (long)(nLength);
                txtReadBytes.Text = mlngReadedBytes.ToString();

                // Save list data
                TechnoAP.usbmca.common.file.WriteBinary(mstrSaveFile, -1, abListData.Length, abListData);
            }

            return nLength;

        }
    }
}
